<?php

namespace Controllers;

use Exception\CustomException;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

class Login
{
    /** @var \Services\Login $loginService */
    protected $loginService;

    /**
     * @param \Services\Login $loginService
     */
    public function __construct(\Services\Login $loginService) {
        $this->loginService = $loginService;
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     * @throws \Exception
     */
    public function login(Request\Filter $request, Response $response) {
        try {
            $conditions    = $request->getFilterCriteria()->getConditions();
            $user          = Utilities::getValueInFilterDataSourceByName($conditions, 'EMAIL');
            $password      = Utilities::getValueInFilterDataSourceByName($conditions, 'PASSWORD');
            $productId     = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID');
            $requesterUrl  = Utilities::getValueInFilterDataSourceByName($conditions, 'REQUESTER_URL');
            $attempts      = Utilities::getValueInFilterDataSourceByName($conditions, 'ATTEMPTS');
            $fullOperator  = Utilities::getValueInFilterDataSourceByName($conditions, 'SHOW_FULL_OPERATOR');
            $hash          = Utilities::getValueInFilterDataSourceByName($conditions, 'HASH');
            $sessionChange = Utilities::getValueInFilterDataSourceByName($conditions, 'SESSION_CHANGE');
            $keepConnected = Utilities::getValueInFilterDataSourceByName($conditions, 'KEEP_CONNECTED');
            $rcUrl         = Utilities::getValueInFilterDataSourceByName($conditions, 'RC_URL');
            $userData      = $this->loginService->login($user, $password, $productId, $requesterUrl, $attempts, $fullOperator, $hash, $sessionChange, $keepConnected, $rcUrl);
            $response->addDataSet(new DataSet ('userData', $userData));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    /**
     * @param Request\DataSet $request
     * @param Response $response
     * @throws \Exception
     */
    public function setLogAccess(Request\DataSet $request, Response $response)
    {
        try{
            $row = $request->getDataSet()->getRows()[0];
            $userId = $row['USER_ID'];
            $this->loginService->setLogAccess($userId);
        }catch (CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     */
    public function validateUser(Request\Filter $request, Response $response) {
        try {
            $conditions         = $request->getFilterCriteria()->getConditions();
            $user               = Utilities::getValueInFilterDataSourceByName($conditions, 'USER');
            $password           = Utilities::getValueInFilterDataSourceByName($conditions, 'PASSWORD');
            $isSupportOperator  = Utilities::getValueInFilterDataSourceByName($conditions, 'IS_SUPPORT_OPERATOR');
            $userValidation= $this->loginService->validateUser($user, $password, $isSupportOperator);
            $response->addDataSet(new DataSet('userValidation', $userValidation));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     */
    public function getProductLanguages(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $productId  = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID');
            $languages  = $this->loginService->getProductLanguages($productId);
            $languages  = empty($languages) ? array() : $languages;
            $response->addDataSet(new DataSet('languages', $languages));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     */
    public function testLicenseConnection(Request\Filter $request, Response $response) {
        try {
            $result = $this->loginService->testLicenseConnection();
            $response->addDataSet(new DataSet('testLicenseConnection', $result));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     */
    public function validateRedirectionUrl(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $url        = Utilities::getValueInFilterDataSourceByName($conditions, 'URL');
            $result     = $this->loginService->validateRedirectionUrl($url);
            $response->addDataSet(new DataSet('validateRedirectionUrl', $result));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }
}