ALTER TABLE RESOURCE_T ADD USR_IN_CHANGING VARCHAR2(01)
/

ALTER TABLE CLIENTE ADD USR_IN_CHANGING VARCHAR2(01)
/


/* ************************************************************************** */
/*                                                                            */
/* Trigger: CLIENTE_TO_RESOURCE_T                                             */
/* Tabela : CLIENTE                                                           */
/*                                                                            */
/* Criao:                                                                   */
/* 10/08/2017 - Newton N. Santos                                              */
/*                                                                            */
/* ************************************************************************** */

CREATE OR REPLACE TRIGGER CLIENTE_TO_RESOURCE_T
BEFORE INSERT OR UPDATE OR DELETE ON CLIENTE
FOR EACH ROW

DECLARE
  V_RESOURCE_T_ID RESOURCE_T.ID%TYPE;	
  V_UPDATED_BY RESOURCE_T.UPDATED_BY%TYPE;	

  V_EXIST_RESOURCE_CLIE BOOLEAN;
  V_EXTERNAL_CODE RESOURCE_T.EXTERNAL_CODE%TYPE;
  
BEGIN
	IF (NVL(:NEW.USR_IN_CHANGING, 'N') <> 'Y') THEN
		IF (INSERTING OR UPDATING) THEN 
			V_EXIST_RESOURCE_CLIE := TRUE;
			BEGIN
				 SELECT EXTERNAL_CODE
					INTO V_EXTERNAL_CODE
					FROM RESOURCE_T
				  WHERE EXTERNAL_CODE = :OLD.CDCLIENTE;
			EXCEPTION
				WHEN NO_DATA_FOUND THEN BEGIN
					V_EXIST_RESOURCE_CLIE := FALSE;
				END;	
			END;
		
			BEGIN
				SELECT ID
				  INTO V_UPDATED_BY
				  FROM RESOURCE_T
				 WHERE EXTERNAL_CODE = :NEW.CDOPERULTATU;
			EXCEPTION
				WHEN NO_DATA_FOUND THEN BEGIN
					V_UPDATED_BY := NULL;
				END;	
			END;
		
			IF (INSERTING OR (NOT V_EXIST_RESOURCE_CLIE)) THEN
				SELECT RESOURCE_T_ID_SEQ.NEXTVAL 
				  INTO V_RESOURCE_T_ID
				  FROM DUAL; 
				  
				INSERT INTO RESOURCE_T (
								ID,
								ORGANIZATION, 
								NAME, 
								RESOURCES_TYPE, 
								CLASS, 
								CALENDAR, 
								EMAIL, 
								PASSWORD, 
								CITY, 
								ZIP_CODE, 
								ADDRESS, 
								DISTRICT, 
								PHONE, 
								NICK_NAME, 
								DOCUMENT_NUMBER, 
								DESCRIPTION, 
								IS_USER, 
								STATUS, 
								CREATED_BY, 
								CREATED_AT, 
								UPDATED_BY, 
								UPDATED_AT, 
								PARENT, 
								EXTERNAL_ID, 
								AVAILABLE_FROM, 
								RATING, 
								EXTERNAL_CODE )      			
					  VALUES (V_RESOURCE_T_ID,                 			/* ID */
								20,                               			/* ORGANIZATION */
								:NEW.NMFANTCLIE,                  			/* NAME */
								'W', 					             				/* RESOURCES_TYPE */
								'S',                             			/* CLASS */
								NULL,                             			/* CALENDAR */
								:NEW.DSEMAILCOMCL,                			/* EMAIL */
								NULL,                             			/* PASSWORD */
								NULL,                             			/* CITY */
								NULL,                             			/* ZIP_CODE */
								NULL,                  				 			/* ADDRESS */
								NULL,                   			 			/* DISTRICT */
								NULL,                  				 			/* PHONE */
								:NEW.NMRAZSOCCLIE,                  		/* NICK_NAME */
								:NEW.NRINSJURCLIE,                			/* DOCUMENT_NUMBER */
								NULL,                							/* DESCRIPTION */
								'N',                              			/* IS_USER */
								DECODE(:NEW.IDCLIEATIV, 'N', 'I', 'A'),   /* STATUS */
								V_UPDATED_BY,                					/* CREATED_BY */
								SYSDATE,                 						/* CREATED_AT */
								V_UPDATED_BY,          							/* UPDATED_BY */
								SYSDATE,                						/* UPDATED_AT */
								NULL,                 							/* PARENT */
								NULL,                							/* EXTERNAL_ID */
								NULL,                							/* AVAILABLE_FROM */
								1,                   							/* RATING */
								:NEW.CDCLIENTE);						 			/* EXTERNAL_CODE */
			END IF;
		
			IF (UPDATING AND V_EXIST_RESOURCE_CLIE)  THEN
				UPDATE RESOURCE_T 
					SET ID 					= ID,
						 ORGANIZATION 		= ORGANIZATION, 
						 NAME 				= :NEW.NMFANTCLIE, 
						 RESOURCES_TYPE 	= RESOURCES_TYPE, 
						 CLASS 				= CLASS, 
						 CALENDAR 			= CALENDAR, 
						 EMAIL 				= :NEW.DSEMAILCOMCL, 
						 PASSWORD 			= PASSWORD, 
						 CITY 				= CITY, 
						 ZIP_CODE 			= ZIP_CODE, 
						 ADDRESS 			= ADDRESS, 
						 DISTRICT 			= DISTRICT, 
						 PHONE 				= PHONE, 
						 NICK_NAME 			= :NEW.NMRAZSOCCLIE, 
						 DOCUMENT_NUMBER 	= :NEW.NRINSJURCLIE, 
						 DESCRIPTION 		= DESCRIPTION, 
						 IS_USER 			= IS_USER, 
						 STATUS 				= DECODE(:NEW.IDCLIEATIV, 'N', 'I', 'A'), 
						 CREATED_BY 		= CREATED_BY, 
						 CREATED_AT 		= CREATED_AT, 
						 UPDATED_BY 		= V_UPDATED_BY, 
						 UPDATED_AT 		= SYSDATE, 
						 PARENT 				= PARENT, 
						 EXTERNAL_ID 		= EXTERNAL_ID, 
						 AVAILABLE_FROM 	= AVAILABLE_FROM, 
						 RATING 				= RATING,
						 USR_IN_CHANGING  = 'Y'
				 WHERE EXTERNAL_CODE = :OLD.CDCLIENTE;
			END IF;
		END IF;
	ELSE
		:NEW.USR_IN_CHANGING := 'N';
	END IF;
	
	IF DELETING THEN
		UPDATE RESOURCE_T 
			SET STATUS 			   = 'D', 
				 EXTERNAL_CODE    = NULL,
				 USR_IN_CHANGING  = 'Y'
		 WHERE EXTERNAL_CODE = :OLD.CDCLIENTE;
	END IF;
END;
/

/* ************************************************************************** */
/*                                                                            */
/* Trigger: RESOURCE_T_TO_CLIENTE                                             */
/* Tabela : RESOURCE_T                                                        */
/*                                                                            */
/* Criao:                                                                   */
/* 11/08/2017 - Newton N. Santos                                              */
/*                                                                            */
/* ************************************************************************** */

CREATE OR REPLACE TRIGGER RESOURCE_T_TO_CLIENTE
BEFORE INSERT OR UPDATE OR DELETE ON RESOURCE_T
FOR EACH ROW

DECLARE
  V_CDCLIENTE CLIENTE.CDCLIENTE%TYPE;	

BEGIN
	IF (DELETING AND (:OLD.RESOURCES_TYPE = 'W') AND (:OLD.CLASS = 'S')) OR
	   ((INSERTING OR UPDATING) AND (:NEW.RESOURCES_TYPE = 'W') AND (:NEW.CLASS = 'S')) THEN

		IF (NVL(:NEW.USR_IN_CHANGING, 'N') <> 'Y') AND (DELETING OR (:NEW.EXTERNAL_CODE IS NULL) OR (:NEW.STATUS = 'D')) THEN
			RAISE_APPLICATION_ERROR(-20161, 'ATENO! A incluso ou excluso de clientes  controlada na TEKNISA e est bloqueada nesse portal. Procure o setor responsvel para solicitar a realizao da modificao desejada.');
		END IF;
		
		V_CDCLIENTE := :NEW.EXTERNAL_CODE;

		IF (NVL(:NEW.USR_IN_CHANGING, 'N') <> 'Y') THEN
			IF UPDATING THEN
				UPDATE CLIENTE 
					SET NMFANTCLIE   = :NEW.NAME, 
						 DSEMAILCOMCL = :NEW.EMAIL, 
						 NMRAZSOCCLIE = :NEW.NICK_NAME, 
						 NRINSJURCLIE = :NEW.DOCUMENT_NUMBER, 
						 IDCLIEATIV   = DECODE(:NEW.STATUS, 'I', 'N', 'S'),
						 USR_IN_CHANGING  = 'Y'
				 WHERE CDCLIENTE = V_CDCLIENTE;
			END IF;
		ELSE
			:NEW.USR_IN_CHANGING := 'N';
		END IF;
	END IF;	
END;
/
